import { AxiosInstance } from 'axios';

declare type Operator = 'eq' | 'not' | 'in' | 'nin' | 'gt' | 'gte' | 'lt' | 'lte' | 'fulltext' | 'match';
declare type QueryValue = string | number | boolean;
declare class Documents {
    private client;
    private _lang?;
    private _limit?;
    private _offset?;
    private _include;
    private _select;
    private _order;
    private q;
    constructor(client: Doclify);
    where(field: string, operator: Operator, value: QueryValue | QueryValue[]): this;
    collection(value: string): this;
    contentType(value: string): this;
    id(value: string): this;
    uid(value: string): this;
    eq(field: string, value: QueryValue): this;
    not(field: string, value: QueryValue): this;
    in(field: string, value: QueryValue[]): this;
    nin(field: string, value: QueryValue[]): this;
    gt(field: string, value: number): this;
    gte(field: string, value: number): this;
    lt(field: string, value: number): this;
    lte(field: string, value: number): this;
    fulltext(field: string, value: string): this;
    match(field: string, value: string): this;
    query(query: (query: Documents) => void): this;
    language(lang: string): this;
    limit(limit: number): this;
    offset(offset: number): this;
    include(fields: string[]): this;
    select(fields: string[]): this;
    orderBy(field: string, asc?: 'asc' | 'desc'): this;
    getParams(params?: {}): Record<string, any>;
    fetch(): Promise<any>;
    paginate(page: number, perPage: number): Promise<any>;
    first(): Promise<any>;
    firstOrFail(): Promise<any>;
    then(resolve: (value: any) => any, reject?: (value: any) => any): any;
}

declare type NodeSerializer = (node: any) => any;
declare type MarkSerializer = (node: any) => any;
declare type Schema = {
    nodes: Record<string, NodeSerializer>;
    marks: Record<string, MarkSerializer>;
};
declare class HTMLSerializer {
    private nodes;
    private marks;
    constructor(schema?: Schema);
    addNode(key: string, schema: NodeSerializer): void;
    addMark(key: string, schema: MarkSerializer): void;
    render(data?: any): string;
    renderNode(item: any): string;
    renderTag(tags: any, ending: string): any;
    renderOpeningTag(tags: any): any;
    renderClosingTag(tags: any): any;
    getMatchingNode(item: any): any;
    getMatchingMark(item: any): any;
}

declare const defaultSerializer: HTMLSerializer;
declare const asHTML: (doc: any) => string;

type index_HTMLSerializer = HTMLSerializer;
declare const index_HTMLSerializer: typeof HTMLSerializer;
declare const index_defaultSerializer: typeof defaultSerializer;
declare const index_asHTML: typeof asHTML;
declare namespace index {
  export {
    index_HTMLSerializer as HTMLSerializer,
    index_defaultSerializer as defaultSerializer,
    index_asHTML as asHTML,
  };
}

declare namespace DOM {
  export {
    index as richText,
  };
}

interface DoclifyOptions {
    url?: string;
    repository?: string;
    key?: string;
    timeout?: number;
    language?: string;
}
interface DoclifyDefaultOptions {
    url?: string;
    repository?: string;
    key?: string;
    timeout: number;
    language?: string;
}

declare class Doclify {
    options: DoclifyDefaultOptions;
    httpClient: AxiosInstance;
    dom: typeof DOM;
    constructor(options?: DoclifyOptions);
    get baseUrl(): string;
    request<T = any>(endpoint: string, options?: Record<string, any>): Promise<any>;
    documents(): Documents;
}

export { DOM, DoclifyDefaultOptions, DoclifyOptions, Doclify as default };
